#####################################################################
#
#  Hornet SQL Bridge (C) 2005 Mooball.com pty ltd
#  A Zope product to mount an SQL Database into the ZODB
#
#  License: ZPL http://www.zope.org/Resources/ZPL
#  Contact: hornet@mooball.com
#
#  Please see http://plone.org/products/hornet
#
#  *WARNING: This product is ALPHA code. Do not expect it to work.*
#
#####################################################################

#####################################################################
#
#    CallbackIterator
#    Author: James Davies (hornet@mooball.com)
#
#    Lazilly maps a method to each element of a sequence
#
#####################################################################

    

from AccessControl import ClassSecurityInfo, allow_module
from Globals import InitializeClass


allow_module("CallbackIterator") 

class CallbackIterator: 
    """
        CallbackIterator
        Iterates over a sequence and wraps each element with the result of the specified method
        
        usage:
            def myWrapperMethod(data):
                return WrapperObject( data )
            
            i = CallbackIterator( largeResultSet, myWrapperMethod )
            
            another example (Using a lambda function to save space):
            
            i = CallbackIterator( largeResultSet, lambda x: WrapperObject( x ) )
            
            
        Essentially, it's a cross between a generator and map()- the advantage being you can lazily 
        seek/slice to any point in the sequence.
    """
    security = ClassSecurityInfo()                
    security.setDefaultAccess('allow')
    
    def __init__(self, source, method):
        self.source = source
        self.iterator = iter( source )
        self.method = method
        
    def next(self):
        row = self.iterator.next()
        return self.method( row )
    
    def __getslice__(self, start, end):        
        return CallbackIterator( self.source[start:end], self.method ) 
    
    def __getitem__(self, item):
        return self.method( self.source[item] )
    
    def __iter__(self):
        return CallbackIterator( self.source, self.method )
    
    def __len__(self):
        return len(self.source)
    
    
InitializeClass(CallbackIterator)