#####################################################################
#
#  Hornet SQL Bridge (C) 2005 Mooball.com pty ltd
#  A Zope product to mount an SQL Database into the ZODB
#
#  License: ZPL http://www.zope.org/Resources/ZPL
#  Contact: hornet@mooball.com
#
#  Please see http://plone.org/products/hornet
#
#  *WARNING: This product is ALPHA code. Do not expect it to work.*
#
#####################################################################

#####################################################################
#
#    HornerFolder
#    Author: James Davies (hornet@mooball.com)
#
#    Allows mounting of a table via the ZMI
#    Uses properties to determine table name, primary key, etc
#
#####################################################################


from Products.PageTemplates.PageTemplateFile import PageTemplateFile


from ZopeSQLObject import ZopeSQLObject, CMFSQLObject
from SQLFolder import SQLFolder
from OFS.SimpleItem import SimpleItem
from OFS.Traversable import Traversable
from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
#from Products.CMFCore.CMFCorePermissions import *
from Permissions import *


class HornetRecord(CMFSQLObject):
    """ A Row from a table """
    meta_type = portal_type = "Hornet Record"
    icon = 'misc_/Hornet/hornetrecord_icon.gif'


    # Factory Type Information; Ugly- This needs to be auto-generated.
    factory_type_information = (
  { 'id'             : 'Hornet Record'
  , 'meta_type'      : 'Hornet Record'
  , 'description'    : 'Hornet Record'
  , 'icon'           : 'hornetrecord_icon.gif'
  , 'product'        : 'Hornet'
  , 'factory'        : 'dummyAdd'
  , 'immediate_view' : 'hornet_record_view'
  , 'actions'        : ( { 'id'            : 'view'
                         , 'name'          : 'View'
                         , 'action': 'string:${object_url}/hornet_record_view'
                         , 'permissions'   : (View,)
                         }
                       , { 'id'            : 'edit'
                         , 'name'          : 'Edit'
                         , 'action': 'string:${object_url}/hornet_record_edit_form'
                         , 'permissions'   : (ModifyPortalContent,)
                         }
                       )
  })


    def SearchableText(self):
        """ Returns the SearchableText for this Record """
        return getattr( self, self.getSearchableTextFieldName() )

    def Title(self):
        """ Returns the title """
        return self.get( self.aq_parent.getTitleFieldName() )
        #return getattr( self, self.aq_parent.getTitleFieldName() )


manage_addHornetFolderForm = PageTemplateFile('www/manage_addHornetFolderForm',globals())

def manage_addHornetFolder(self, id, title='', REQUEST=None, **kw):
    """ Creates a new PersonFolder """

    if REQUEST:
        get = lambda x,y=None: REQUEST.get(x,kw.get(x,y))
    else:
        get = lambda x,y=None: kw.get(x,y)

    o = HornetFolder(id, title)
    o.tableName               = get('tableName')
    o.primaryKey              = get('primaryKey')
    o.titleFieldName          = get('titleFieldName', o.primaryKey)
    o.searchableTextFieldName = get('searchableTextFieldName', o.titleFieldName)

    self._setObject(id, o)
    if REQUEST:
        return REQUEST.response.redirect(self.absolute_url() + '/manage_workspace')

def manage_addHornetRecord(self, id, title=''):
    """ Creates a new PersonFolder """
    raise "Not Implemented"
    #o = HornetRecord(id, title)
    #self._setObject(id, o)


class HornetFolder(SQLFolder):
    """ Hornet Folder """
    meta_type = "Hornet Folder"
    icon = 'misc_/Hornet/hornetfolder_icon.gif'

    klass = HornetRecord

    titleFieldName   = 'title'
    tableName        = 'content'
    searchableTextFieldName = 'text'

    _properties=({'id':'title',                   'type': 'string', 'mode': 'w'},
                 {'id':'primaryKey',              'type': 'string', 'mode': 'w'},
                 {'id':'titleFieldName',          'type': 'string', 'mode': 'w'},
                 {'id':'searchableTextFieldName', 'type': 'string', 'mode': 'w'},
                 {'id':'tableName',               'type': 'string', 'mode': 'w'},
                )

    def getSearchableTextFieldName(self):
        return self.searchableTextFieldName

    def getTitleFieldName(self):
        return self.titleFieldName

    def getTableName(self):
        """ """
        return self.tableName

    factory_type_information = (
  { 'id'             : 'Hornet Folder'
  , 'meta_type'      : 'Hornet Folder'
  , 'description'    : 'Hornet Folder'
  , 'icon'           : 'hornetfolder_icon.gif'
  , 'product'        : 'Hornet'
  , 'factory'        : 'manage_addHornetFolder'
  , 'immediate_view' : 'folder_contents'
  , 'actions'        : ( { 'id'            : 'folder_contents'
                         , 'name'          : 'Folder Contents'
                         , 'action': 'string:${object_url}/folder_contents'
                         , 'permissions'   : (View,)
                         }
                       , { 'id'            : 'edit'
                         , 'name'          : 'Edit'
                         , 'action': 'string:${object_url}/hornetFolder_edit_form'
                         , 'permissions'   : (ModifyPortalContent,)
                         }
                       ,
                         { 'id'            : 'view'
                         , 'name'          : 'View'
                         , 'action'        : 'string:${object_url}/view'
                         , 'permissions'   : (View,)
                         }
                         
                       )
  })



InitializeClass(HornetRecord)
InitializeClass(HornetFolder)
