#####################################################################
#
#  Hornet SQL Bridge (C) 2005 Mooball.com pty ltd
#  A Zope product to mount an SQL Database into the ZODB
#
#  License: ZPL http://www.zope.org/Resources/ZPL
#  Contact: hornet@mooball.com
#
#  Please see http://plone.org/products/hornet
#
#  *WARNING: This product is ALPHA code. Do not expect it to work.*
#
#####################################################################

#####################################################################
#
#    SQLFolder
#    Author: James Davies (hornet@mooball.com)
#
#    Mounts an SQL Table into the ZODB
#    Most of this just implements the standard Zope Folder interface
#
#####################################################################


# TODO: Document this. 


# Zope Imports
from OFS.ObjectManager import ObjectManager
from OFS.Traversable import Traversable
from AccessControl import ClassSecurityInfo
from Products.PageTemplates.PageTemplateFile import PageTemplateFile 


# Hornet Imports
from SQLIterator import SimpleSQLIterator
from SQLObject import SQLObject
from CallbackIterator import CallbackIterator
from FolderMixin import BaseFolder
from Schema.DisplayList import DisplayList
from cStringIO import StringIO

_marker = "_marker"

class SQLFolder(SimpleSQLIterator, BaseFolder, Traversable):
    
    primaryKey = 'id'
    
    isHornetFolder = True
    isPrincipiaFolderish = 1
    
    manage_main  = PageTemplateFile('www/paged_manage_main',globals())
    manage_tools = PageTemplateFile('www/manage_tools',globals())

    
    
    manage_options=(
        (
            {'label':'Contents', 'action':'manage_main',}, 
            {'label':'View', 'action':''},
            {'label':'Properties','action':'manage_propertiesForm'},
            {'label':'Tools','action':'manage_tools'},
        ))
    
    def __init__(self, id='', title='', query=None):
        self.id = id
        self.title = title    
        SimpleSQLIterator.__init__(self, query) 
        BaseFolder.__init__(self, id)

    security = ClassSecurityInfo()
    security.setDefaultAccess('allow')
    security.declareObjectPublic()        
    
    security.declareProtected('List folder contents','listFolderContents') 
    def listFolderContents(self, contentFilter=None):
        """ CMF Aware iterator """
        
        # 'getFolderListingFolderContents' passes us an explicit acquisition wrapper, 
        # which farks things up big time. Rewrap ourself with an implicit wrapper.
        
        implicit = self.aq_base.__of__( self.aq_parent )
        return implicit.objectValues() 
        #return self.aq_self.objectValues() 

    def getDescription(self):
        """ Returns the table/query structure """
        query = self.getQuery(noCustom=True)
        query += " LIMIT 0"
        rs = self.sqlQuery( query )
        return rs.describe()
        #try:return rs.describe()
        #except Exception, e:
        #    print str(e)
            

    
    def createEmpty(self):
        """ Creates an empty object. """
        description = self.getDescription() 
        #description = self.describeTable( self.getTableName() )
        data = {}
        for field in description:
            if type(field) is dict:
                key = field['name']
            if type(field) is tuple:
                key = field[0]
            
            data[key] = None
        object = self.wrapObject( data ) 
        object._new = True
        return object
        
        
        
    def __getitem__(self, key):        
        
        if key == 0: 
            raise "Trying to get key 0"

        # If the key is 'create' (i.e. http://server/mountpoint/table/create)
        # then return an empty object.
        # -- This is a bit of a hack, but works quite well. 
        if key == 'create':
            return self.createEmpty()
              
        # Check if the key is a number; if so, assume we need to grab it from the database.
        # (Otherwise, pass on to ObjectManager)               
        
        #FIXME: Need to be able to handle non-numeric primary keys
        try:
            if str(int(key)) == str(key):
                data = self.getById(key)
                if not data: return None
                o = self.wrapObject( data )
                return o
        except ValueError:
            return ObjectManager.__getitem__(self, key)
        #except Exception, e:
        #    print "getitem error: " + str(e)

    def getById(self, key):
        if hasattr(self.aq_explicit, 'customRowQuery'):
            rs = self.sqlQuery( self.customRowQuery % `key`)
            return rs[0]
            
        else:
            return self.sqlRowById(self.getTableName(), key, primary=self.getPrimaryKey())

    security.declareProtected('Access contents information','objectItems')
    def objectItems(self, filter=None):        
        for row in self.objectValues(filter):
            yield (row.getId(), row)
            
    def Like(self, key, value, start=None, end=None):
        table = self.getTableName()
        sql = "SELECT * FROM %s WHERE %s LIKE '%s' ORDER BY %s" % (table, key, value, key)
        #if limit: sql += " LIMIT %s" + str(limit)
        
        return self.wrapResults( self.sqlQuery(sql) )
    
    def wrapObject(self, row):
        """ Wraps the specified row in the relevant object for this table/folder """            
        
        # Allow subclasses to modify the row before wrapping it (i.e.- to convert char encoding)
        if hasattr(self, '_preprocess_row_load'): row = self._preprocess_row_load(row)
        
    
        # Wrap the row in the correct object, then wrap in our aqcuisition layer
        o = self.klass(row).__of__(self)
        
        # When the Zope transaction machinery calls our commit() method, we
        # don't have access to the acquisition wrapper. 
        # Make sure we have a reference to the folder and mountpoint!
        # NOTE: Double check for potential memory leaks
        o._v_mountpoint = self.getMountPoint()
        o._v_folder     = self
        
        # Used to explicitly mark this object as New or Existing
        o._v_new        = None
        
        return o
            
    def getPrimaryKey(self):
        """ Returns the primary key """
        return self.primaryKey   
    
    security.declareProtected('Manage portal','fromQuery') 
    def fromQuery(self, query):
        """ Executes the specified SQL query and returns the 
            results wrapped in transient objects """
        return self.wrapResults( self.sqlQuery( query ))    
        #for row in self.sqlQuery( query ):
        #    yield self.wrapObject( row ) 
            
    def sqlSearch(self, dictQuery=None, sort=None, **query):
        if dictQuery: query = dictQuery
        
        # Mimick ZCatalog. (FIXME: Need to mimick a lot more than sort_on....)
        if 'sort_on' in query:
            sort = query['sort_on']
            del query['sort_on']
        
        results = self.sqlSearchTable(self.getTableName(), query, sort)        
        return self.wrapResults( results )
        
    def wrapResults(self, results):
        """ Wraps each element in the sequence with transient objects """
        return CallbackIterator( results,  lambda x: self.wrapObject( x ))
    
    security.declareProtected('Access contents information','objectValues')
    def objectValues(self, filter=None):
        """ Returns an iterable sequence of objects in this folder """
        
        return self.wrapResults( self.Iterator() )
            
        #for row in self.Iterator(): 
        #   yield self.wrapObject( row )
            
    def getParentNode(self):
        """ Returns the parent folder (used by plone) """
        return self.aq_parent            
            
    def contentIds(self, spec=None, filter=None):
        return self.objectIds( filter )
    
    security.declareProtected('Access contents information','objectIds')    
    def objectIds(self, filter=None):
        
        
        return self._fetchObjectIds(filter)
        
        #if hasattr(self, '_v_cache_ids'):
        #    return self._v_cache_ids    
        #else:
        #    return self._fetchObjectIds(filter)
        
    def _fetchObjectIds(self, filter=None):
        cache_ids = []
        for row in self.Iterator():
            id = row[self.getPrimaryKey()]
            cache_ids.append( id )
            yield id
            
        self._v_cache_ids = cache_ids
        
                    
    def browserDefault(self, **kw):
        """ Another method to make Zope think we're a real object """
        return ((), ())    
        
    def manage_fixupOwnershipAfterAdd(self):
        """ Fixup ownership. Not really needed as we ignore permissions (for now?) """
        pass              
        
    def manage_afterAdd(self, item, container):
        """ """
        pass        
    
    security.declareProtected('Modify portal content', 'deleteRecords')
    def deleteRecords(self, ids=[]):
        #FIXME: Potential SQL Injection exploit. Need to verify 'ids'
        
        id_list = ','.join(ids)
        
        table      = self.getTableName()
        primaryKey = self.getPrimaryKey()
        
        query = "DELETE FROM %s WHERE %s IN (%s)" % \
                    (table, primaryKey, id_list)
                    
        self.sqlQuery(query)
        
    security.declareProtected('Modify portal content','manage_delObjects')
    def manage_delObjects(self, ids=[], REQUEST=None): 
        """ Deletes the specified items from the database """
        if type(ids) is type(''): ids=[ids]
        if not ids:
            return MessageDialog(title='No items specified',
                   message='No items were specified!',
                   action ='./manage_main',)
                   
        
        self.deleteRecords(ids)
        
        
        
        if REQUEST is not None:
            return self.manage_main(self, REQUEST, update_menu=1)
        
    #ecurity.declareProtected('List folder contents','folder_contents')    
    #ef folder_contents(self, *args, **kwargs):
    #   """ Return our custom folder contents """
    #    return self.hornet_folder_contents(*args, **kwargs)
    
    def getTypeName(self):
        """ Returns the meta type to use """
        return self.klass.portal_type
    
    def createObject(self, *args, **kwargs):
        """ Fake createObject method that redirects us to the default edit view of a transient object """
        obj = self.createEmpty()
        action = obj.getTypeInfo().getActionById('edit')
        url = obj.absolute_url() + '/' + action
        
        return self.REQUEST.RESPONSE.redirect(url)

    def allowedContentTypes( self ):
        """ Returns the allowed content types """
        obj = self.createEmpty()
        type = obj.getTypeInfo()
        if type.isConstructionAllowed(self):
            return (type,)  
        return ()
        
    # Deprecated
    def keyValuePairs( self, field, key=None, sort=None ):
        return self.sqlDistinctField( self.getTableName(), field, key, sort )
        
    
    def createVocab( self, key, label, sort=None ):
        return DisplayList( self.sqlDistinctField( self.getTableName(), label, key, sort ))  

            
    def reindexChildren( self ):
        """ (Re)indexes the contents of this folder """
    
        out = StringIO()    
        count = 0
        for child in self.objectValues():
            child.reindexObject()
            print >> out, child.getId()
            
            count+=1
            if count % 100 == 0:
                get_transaction().commit()
                print count
            
        return out.getvalue()
        
    def getTypeInfo(self):
        """ Returns the correct type info object """
        
        result = self.portal_types.getTypeInfo(self.meta_type)
        return result
        
    
    