#####################################################################
#
#  Hornet SQL Bridge (C) 2005 Mooball.com pty ltd
#  A Zope product to mount an SQL Database into the ZODB
#
#  License: ZPL http://www.zope.org/Resources/ZPL
#  Contact: hornet@mooball.com
#
#  Please see http://plone.org/products/hornet
#
#  *WARNING: This product is ALPHA code. Do not expect it to work.*
#
#####################################################################

#####################################################################
#
#    SQLMountPoint
#    Author: James Davies (hornet@mooball.com)
#
#    The root container of 'table' objects.
#    contains numerous utility methods accessed via acquisition 
#
#####################################################################

# SQL Mount Point.  


from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from Globals import InitializeClass
from RowSource import RowSource
from Shared.DC.ZRDB.Results import Results
from DateTime import DateTime
from FolderMixin import BaseFolder
from Permissions import *
from Products.CMFCore.PortalContent import PortalContent
from AccessControl import ClassSecurityInfo
from OFS.ObjectManager import ObjectManager

from WizardMixin import WizardMixin

import re



def escape( string, quote=True ):

    # HACKHACK- old method of escaping strings was messing up unicode chars.
    # Do it manually for now, find a cleaner way later.
        
    if string is None:
        return 'null'
    #rendered = ("%s" % string)
    string = "%s" % string
    for char in ("\\",'"',"'"):
        string = string.replace(char, '\\' + char)
        
    if quote:return "'%s'" % string 
    else:    return string
    
    #return "'%s'" % ( re.escape( str(string) ))


manage_addHornetMountPointForm = PageTemplateFile('www/manage_addHornetMountPointForm.zpt', globals())

def manage_addHornetMountPoint(self, id, REQUEST=None):
    """ Creates a new Author Folder """
    o = SQLMountPoint(id)
    self._setObject(id, o)
    if REQUEST:
        return REQUEST.response.redirect(self.absolute_url() + '/manage_main')

def manage_addCMFSQLMountPoint(self, id):
    """ Creates a new Author Folder """
    o = CMFSQLMountPoint(id)
    self._setObject(id, o)


class SQLMountPoint(BaseFolder, WizardMixin, PortalContent):
    """ SQL Mount Point; Contains SQL Folders """
    meta_type = portal_type = "Hornet Mount Point"
    allowed_content_types = ('Hornet Folder',)
    icon = 'misc_/Hornet/hornetmountpoint_icon.gif'

    manage_options=(
        (
            { 'label':'Contents',   'action':'manage_main',          },
            { 'label':'View',       'action':''                      },
            { 'label':'Properties', 'action':'manage_propertiesForm' },
            { 'label':'Setup',      'action':'manage_wizard'         },
        ))

    # id of the Z SQL Connector
    _connectionId = 'sql_connection'

    security = ClassSecurityInfo()
          
    
    def getMountPoint(self):
        """ Returns a reference to the mount point (This object) """
        return self

    def getAdapter(self):
        """ Gets a reference to the SQL Adapter """
        #from FastMysql import FastMysql
        #return FastMysql()
        return getattr(self, self.getConnectionId())()
        #return self.sql_connection()

    def isAdapterValid(self):
        try:
            self.getAdapter()
            return True
        except AttributeError:
            return False


    def sqlDistinctField(self, table, field, key=None, sort=None):
        if not key: key = field
        query = "SELECT DISTINCT %s as `key`, %s as `value` FROM %s" % (key, field, table)
        if sort:
            query += " ORDER BY %s" % sort
        return self.sqlQuery( query )

    security.declareProtected('Manage portal','sqlQuery')
    def sqlQuery(self, query):
        """
            Execute the specified query, returning a RowSource object containing the results
            (If no results are returned, return an empty tuple) 
        """

        #print "Executing query: \n\n" + str(query) 
        #print "========================================================="
        #import traceback
        #traceback.print_stack()
        #print "========================================================="

        adapter = self.getAdapter()

        this = self.aq_explicit
        if not hasattr( self, '_v_set_charset' ) and hasattr( self, 'set_charset' ):
            adapter.query('SET NAMES ' + getattr(this, 'set_charset'))
            self._v_set_charset = True


        t = DateTime()
        try:desc, results = adapter.query( query, max_rows = 0, exposeCursor=True )
        except TypeError:
            #print "Not using Moo ZMySQLDA? Falling back to default"
            desc, results = adapter.query( query, max_rows = 0 )

        #print "time taken: " + str((DateTime() - t) * 24 * 60 * 60)

        if not desc:
            # We did not perform a select query, so assume we did an insert.

            # FIXME: NOT THREAD SAFE!
            # Get insert_id() from the connection
            if type(results) is tuple:
                insert_id = adapter.db.insert_id()
            else:
                insert_id = results.lastrowid

            return insert_id



        from CallbackIterator import CallbackIterator
        from RowSource import buildDict

        if type(results) is tuple:
            cb = CallbackIterator( results, lambda row: buildDict( row, desc ))
            cb.description = desc
            cb.describe = lambda: desc
            return cb

        else:
            wrapper = RowSource( results )
            return wrapper


    def sqlRowById(self, table, id, primary='id'):
        """
            Return the row from the specified table where the primary key == id
            If no results are found, return None
        """
        query = "SELECT * from %s WHERE %s='%s'" % (table, primary, id)
        rs = self.sqlQuery(query)
        if not rs: return None
        return rs[0]


    def sqlInsert( self, table, kwdata=None, **kw ):

        l_keys   = []
        l_values = []
        if not kwdata: kwdata = kw
        for key, value in kwdata.items():
            l_keys.append  ( "`%s`" % key)
            l_values.append( escape(value) )

        keys   = ','.join( l_keys )
        values = ','.join( l_values )

        query = "INSERT INTO %(table)s (%(keys)s) VALUES (%(values)s)" % locals()
        return self.sqlQuery( query )


    def sqlUpdate( self, table, delta, primaryKey='id', primaryValue = None ):

        if not delta:
            return

        #table = escape( table )
        #primaryKey = escape( primaryKey )

        if not primaryValue: primaryValue = delta[primaryKey]

        primaryValue = escape( primaryValue )

        l_pairs = []
        for (key, value) in delta.items():
            l_pairs.append( "`%s`=%s" % (key, escape(value)) )

        pairs = ','.join(l_pairs)

        query = "UPDATE %(table)s SET %(pairs)s WHERE %(primaryKey)s = %(primaryValue)s LIMIT 1" % locals()
        self.sqlQuery( query ) 

    def sqlSearchTable( self, table, query=None, order=None, **kw ):

        if not query: query = kw
        where = ""
        for key, value in query.items():
            if not key or not value: continue
            value = escape(value)
            where += "AND `%s` = %s " % (key, value)

        sql = "SELECT * FROM %(table)s WHERE 1 %(where)s " % locals()
        if(order):
            sql += "ORDER BY %(order)s" % locals()

        return self.sqlQuery( sql )

    def sqlDelete( self, table, limit=None, query=None, **kw):
        where = ""
        if not query: query = kw
        for key, value in query.items():
            if not key or not value: raise "Empty key/value in deletion filter"
            value = escape(value)
            where += "AND `%s` = %s " % (key, value)

        sql = "DELETE FROM %(table)s WHERE 1 %(where)s " % locals()
        if limit:
            sql += " limit %s" % `limit`

        return self.sqlQuery( sql )

    def describeTable( self, table ):
        """
            Returns the column information for the specified table
        """
        rs = self.sqlQuery("DESCRIBE %s" % escape(table, False))
        return rs
        #         rs = self.sqlQuery( "SELECT * FROM %s LIMIT 0" % table )
        #         d = rs.describe() 
        #         print `d`
        #         return d
        
        
    def manage_delObjects(self, ids=[], REQUEST=None):
        """ """
        pm = getattr(self, 'portal_membership', None)
        if pm: # We are in plone
            return PloneFolder.manage_delObjects( self, ids, REQUEST=REQUEST )
        else:
            return ObjectManager.manage_delObjects( self, ids, REQUEST=REQUEST )
            


    factory_type_information = (
  { 'id'             : 'Hornet Mount Point'
  , 'meta_type'      : 'Hornet Mount Point'
  , 'description'    : 'Mounts an SQL Database into the ZODB'
  , 'icon'           : 'hornetmountpoint_icon.gif'
  , 'product'        : 'Hornet'
  , 'factory'        : 'manage_addHornetMountPoint'
  , 'immediate_view' : 'folder_contents'
  , 'actions'        : ( { 'id'            : 'view'
                         , 'name'          : 'Contents'
                         , 'action': 'string:${object_url}/folder_contents'
                         , 'permissions'   : (View,)
                         },
                         { 'id' : 'setup',
                           'name': 'Setup (ZMI)',
                           'action': 'string:${object_url}/manage_wizard',
                         }
                       )
  })



InitializeClass(SQLMountPoint)
#InitializeClass(CMFSQLMountPoint)
