#####################################################################
#
#  Hornet SQL Bridge (C) 2005 Mooball.com pty ltd
#  A Zope product to mount an SQL Database into the ZODB
#
#  License: ZPL http://www.zope.org/Resources/ZPL
#  Contact: hornet@mooball.com
#
#  Please see http://plone.org/products/hornet
#
#  *WARNING: This product is ALPHA code. Do not expect it to work.*
#
#####################################################################

#####################################################################
#
#    SQLTransactionAware
#    Author: James Davies (hornet@mooball.com)
#
#    Integrates our SQL functionality with the Zope transaction machinery
#
#    Based on http://mail.zope.org/pipermail/zodb-dev/2001-July/001180.html
#    And code from the ZMysqlDA adapter
# 
#####################################################################
 




class Surrogate:
    def __init__(self, db):
        self._p_jar=db

class SQLTransactionAware:
    """Mix-in class that provides transaction management support

    A sub class should call self._register() whenever it performs
    any transaction-dependent operations (e.g. sql statements).

    The sub class will need to override:
      _begin if necessary
      _vote to raise an except if necessary
      _finish to finallize work,
      _abort to roll-back work
      _postCommit to guaruntee all commit processing is done

    This is similiar in nature to TM.py and some ZPatterns code.
    """
    
    _registered = None
    
    def _register(self):
        #print "Registering..."
        if not self._registered: 
            try:
                #print "w00t"
                get_transaction().register(Surrogate(self))
                self._begin()
                self._registered = 1
                #self._finalize = 0
            except Exception, e: 
                print "Error registering: " + str(e)
                raise

    def sortKey(self, *ignored):
        """ The sortKey method is used for recent ZODB compatibility which
            needs to have a known commit order for lock acquisition.  Most
            DA's talking to RDBMS systems do not care about commit order, so
            return the constant 1
        """
        return 1

    def tpc_vote(self, *args):
        pass


    def tpc_begin(self, *args):
        pass
        #print "tpc_begin()"
        
    def tpc_finish(self, *args):
        self.sql_commit()
        #print "tpc_finish()"
        
    def tpc_abort(self, *args):
        self.abort( *args ) 
        
    def commit(self, *args):
        pass
        #print "commit()"
        
    
    def _begin(self, *args):
        """Hook method to begin external transaction.
        This may be called multiple times,"""
        #print "_begin()"
        pass

    def _vote(self, *args):
        """Hook method to vote on success of transaction commit.
        This will only be called once,"""
        #print "_vote()"
        pass

    def _finish(self, *args):
        """Hook method to complete transaction work.
        This may be called multiple times."""
        #print "_finish()"
        pass

    def _abort(self, *args):
        """Hook method to undo transaction work.
        This may be called multiple times."""
        #print "_abort()"
        pass
    
    def abort(self, *args):
        self._abort(*args)

    def _postCommit(self, *args):
        """Hook method called when Transaction has completed a
        commit process.
        """
        pass
        #print "_postCommit()"
        
        
        
        
'''    
http://mail.zope.org/pipermail/zodb-dev/2003-April/004929.html    
        
tpc_begin - indicates the beginning of two-phase commit.  Sent from a 
transaction to all data managers during a transaction.commit().

tpc_abort - indicates that two-phase commit failed, and the transaction 
must be rolled back.  Sent by a transaction to all data managers that it 
sent tpc_begin to, if the commit process fails.  Note that a DM may receive 
abort(ob) calls in addition to this message, or in place of it (if 
tpc_begin wasn't yet called on the DM when the abort began, or if the abort 
didn't occur as a result of an error in the commit process).

tpc_vote - this is the last chance for a data manager to back out of 
committing.  If the DM can't absolutely guarantee it will successfully 
commit, it should raise an error message.  This is sent by the transaction 
to all participating data managers, after tpc_begin and any commit(ob) 
messages.

tpc_finish - go ahead and actually commit.  Sent by the transaction to all 
participating data managers if and only if all tpc_vote calls returned 
successfully.  tpc_finish must not raise any exceptions, or the system will 
enter the "hosed" state.  :)

commit(ob) - sent to a data manager by the transaction (after tpc_begin) to 
indicate that 'ob' is a changed object the DM is responsible for committing.

abort(ob) - sent to a data manager by the transaction to indicate that 'ob' 
is an uncommitted object whose state should be discarded.  This method is 
only called for objects that were not yet passed to the DM's 'commit()' method.        
        
        
'''