from AccessControl import ClassSecurityInfo
from Acquisition import Explicit, Implicit
from Globals import InitializeClass
from Products.Hornet.Schema.Exceptions import NodeNotFound, EndOfGroup

#### Experimental
class Node(Explicit): 
    security = ClassSecurityInfo()                
    security.setDefaultAccess('allow')
    def __init__(self, id, children=(), fieldsets=(), **kw):
        self.id = id
        
        d_children = {}
        for child in children:
            d_children[child.getKey()] = child
        
        
        self.d_children = d_children
        self.children = [child.getKey() for child in children]
        self.fieldsets = fieldsets
        self.kwargs = kw
    
    def walk(self, ignoreRepeat=False):
        ''' Recursively walks the entire tree '''
        
        if ignoreRepeat:
                repeat = self.getArg('repeat')
                if repeat: raise StopIteration
        
        for child in self.getChildren():
            child = child.__of__(self)
            yield child
            
            for c in child.walk( ignoreRepeat=ignoreRepeat ):
                yield c
         
            
        
    def isRoot(self):
        ''' Is this node a root node (no parents)? '''
        return not hasattr(self, 'aq_parent')
        
        
    def isLeaf(self):
        ''' Is this a leaf node (no children) ''' 
        print 'isLeaf( %s ): %s' % (`self`,`self.children`)
        if self.children:
            print 'False'
            return False 
        else:
            print 'True'
            return True   
        
    def findLeaf(self):
        ''' Walk down the tree until we hit a leaf node '''
        if self.isLeaf():
            print `self` + ' is a leaf node '
            return self
        for child in self.getChildren():
            node = child.findLeaf()            
            if node:
                return node.__of__(self) 
    
    def getParent(self):
        return self.aq_parent
    
    def nodePath(self):
        if self.isRoot():
            return [self]
        else:
            return self.getParent().nodePath() + [self] 
        
    def nodePathNames(self):
        path = self.nodePath()
        return [node.getKey() for node in path]
    
    def getChildren(self):
        ''' Returns a sequence of this nodes direct children '''
        for child in self.children:
            yield self.d_children[ child ].__of__(self)
        
    def getFieldsets(self):
        ''' Returns a sequence of fieldset names for this node '''
        return self.fieldsets
            
    def getKey(self):
        return self.id
        
    def getArg(self, key, default=None):
        return self.kwargs.get(key, default)  
    
    def find(self, key):
        print 'Looking for %s in %s' % (`key`, `self.getKey()`) 
        if self.id == key: return self
        for child in self.getChildren():
            found = child.__of__(self).find(key)
            #found = child.find(key)
            if found: 
                return found
       
    def Title(self):
        return self.getArg('title')
                 
    def next(self, key):
        ''' 
            Returns the next node after 'key' in it's group.
            If we hit the end if this group, raise a EndOfNode exception.
            if the key isn't found, raise a NodeNotFound
            The controller is responsible for handling what to do next (i.e. moving to the next group)
        '''
           
        #print 'looking for %s in %s' % (key, self.children)
        
        try:
            index = self.children.index( key )
        except ValueError:
            print self.children
            raise NodeNotFound, 'Cannot find node %s' % `key`
            
        
        try:
            next  = self.children[index + 1]
        except IndexError:
            raise EndOfGroup
        

        nextNode = self.d_children[next]
            
        return nextNode.__of__(self)
        
        
    
class Page(Node):
    security = ClassSecurityInfo()                
    security.setDefaultAccess('allow')
        
    
class PageGroup(Node):
    security = ClassSecurityInfo()                
    security.setDefaultAccess('allow')
            
InitializeClass(Node)
InitializeClass(Page)
InitializeClass(PageGroup) 