from AccessControl import ClassSecurityInfo
from Globals import InitializeClass
from Products.Hornet.Schema.Exceptions import NodeNotFound, EndOfGroup, DatasourceError

def getOrCreate( dictionary, key, value=None ):
    if not value: value = {}
    if not dictionary.has_key( key ):
        dictionary[key] = value
    return dictionary[key]

def get( root, key ):
    try:
        return root[key]
    except KeyError:
        return getattr( root, key )
        
def traverse( context, path ):
    for elem in path.split('/'):
        context = get( context, elem )
    return context
    

def getNamespace( targetNode, state, data ):  
    path = targetNode.nodePath() 
    dbg_path = ''
    datasource = data 
    for node in path:
        key = node.getKey()
        datasource = getOrCreate( datasource, key )
        dbg_path += '/' + key
        
        repeat = node.getArg('repeat')
        if repeat:
            (iterator, sequenceName) = repeat.split(' ')
            if iterator in state:
                num = state[iterator] 
                datasource = getOrCreate( datasource,  num )
                dbg_path += '/' + str(num)
            else:
                sequence = traverse( data, sequenceName )     
                if sequence:
                    print "Got sequence " + `sequence`
                    keys = sequence.keys()
                    keys.sort()
                    state[iterator] = keys[0]
                                
        #datasource = datasource[key]
    
    return datasource
    
    
class SimpleController:
    """ 
        Simple controller object 
        Supports recursive nodes and simple repeats
    """
    
    security = ClassSecurityInfo()                
    security.setDefaultAccess('allow')
        
    def __init__(self, pages, state, datasource):
        self.pages      = pages        
        self.state      = state.copy() 
        self.datasource = datasource
        
    ###########################################################################
        
    def currentNode(self):
        """ Returns the current page object """
        if not self.state.has_key('position'):
            return None
        
        return self.pages.find( self.state['position'] )
        
    
    def rootNode( self ):
        return self.pages
    
    ###########################################################################
    
    def initialize(self):
        """ Called before the first page is loaded """
        
        state     = self.state
        
        first = self.pages.findLeaf()
        #ns = getNamespace( first, state, self.datasource )
        #ns['_available'] = True
        
        namespace = self.datasource
        # Walk the entire tree and setup any repeat variables
        for page in self.pages.walk( ignoreRepeat=True ):
            
            getNamespace( page, state, self.datasource )
                
            
            #if repeat:
            #    iteratorName, countName = repeat.split(' ')
            #    state[iteratorName] = 1
                
        
                
        namespace['_initialized'] = True
        
        return namespace

    ###########################################################################
    

    
    def next( self ): 
        
        state = dict( self.state )
        
        # Get the current node
        current = self.currentNode()
        
        if current:
            # Find the next node to display. 
            next, state = self.findNext( current, state )
        else:
            # If no page is active, get the root node
            next = self.pages 
        
        # Now walk down the tree till we hit a leaf node
        leaf = next.findLeaf()
        
        # Grab the relevant namespace for the new page
        namespace = getNamespace( leaf, state, self.datasource )
        
        self.markEnabled( leaf, state )    
        #namespace['_available'] = True
        
        #print 'STATE: ' + `state`
        
        # Put the new page name into the state variable
        state['position'] = leaf.getKey() 
        
        prepopulate = leaf.getArg('prepopulate')
        if prepopulate:
            prepopulate( leaf, state, namespace, self.datasource ) 
        
        # Return the new node, state, and namespace
        return leaf, state, namespace
        
        
    def current( self ):
        current = self.currentNode() or self.rootNode()
        leaf = current.findLeaf() 
        state = dict( self.state )
        self.markEnabled( leaf, state ) 
        state['position'] = leaf.getKey() 
        
        namespace = getNamespace( leaf, state, self.datasource )  
        
        return leaf, state, namespace
        
    def markEnabled( self, node, state ):
    
        print "Marking %s as enabled" % node.getKey()
    
        namespace = getNamespace( node, state, self.datasource )  
        namespace['_available'] = True
        
        if node.isRoot(): return
        
        self.markEnabled( node.getParent(), state )
        
        
            
    def process(self, request):
        """ Process the request """
        
        state = self.state         
        current = self.currentNode()
        if current is None:
            # This is the first time we've loaded the wizard
            # ... So grab the root node; findLeaf will then find the first page to display.
            self.initialize()
            
        else:
            # We're submitting an existing form. 
            # Check to make sure our datasource is still valid (in case of session timeouts, etc)
            if not self.datasource.has_key('_initialized'):
                raise DatasourceError, \
                    'The Datasource is not initialized' 
            
            namespace = getNamespace( current, state, self.datasource ) 
            
            form_handler = current.getArg('form_handler')
            if form_handler:
                form_handler( request=request, namespace=namespace, datasource=self.datasource, 
                              node=current, state=state )
            
            namespace.update( request )
            
        
        
    
    #################################################################
        
    def findNext(self, current, state):
        """ Finds the next node after 'current' """
        
        if current.isRoot():
            raise EndOfGroup
    
        parent  = current.getParent()
        next = None
        try:
            # Ask the current nodes parent for the next node in it's group
            next = ( parent.next( current.getKey() ), state )
        except EndOfGroup:
            # No more nodes in this group. 
            
            repeat = parent.getArg('repeat') 
            if repeat:
                # This group has a repeat statement; process it
                iteratorName, countName = repeat.split(' ')
                
                #count    = int(traverse( self.datasource, countName )) # The number of times to iterate
                
                sequence = traverse( self.datasource, countName)
                keys     = sequence.keys()
                keys.sort()  
                
                position = state[iteratorName]  # The current iterator position
                index    = keys.index( position )
                
                try:
                    nextKey = keys[index + 1]
                    #next = sequence[ nextKey ]
                    state[iteratorName] = nextKey
                    print "Setting state: " + `state`
                    next =  ( parent, state )
                except IndexError:
                    pass # We've hit the end of the repeat. continue and find the next node.
                    
                #if iterator < count:  # We still have more loops to go
                #    iterator += 1
                #    state[iteratorName] = iterator
                #    return parent, state      # Return the group to repeat. (Leaf will be found later)
                    
        ## END Except Block
                 
        if not next: 
            # No [more] repetition needed; go to the next group
            return self.findNext( parent, state )
        else:
            condition = next[0].getArg('condition') 
            if condition:
                result = condition( node=next[0], state=next[1], datasource=self.datasource ) 
                if not result:
                    return self.findNext( next[0], next[1] )
            return next
            
            
                
                
        
InitializeClass(SimpleController)