from Acquisition import Implicit
from ExtensionClass import Base
from AccessControl import ClassSecurityInfo
from BaseSchema import BaseSchema
from cStringIO import StringIO

_marker = '_marker'

class DictWrapper(Base, Implicit):
    ''' 
        Extension class to wrap dictionaries; allows PySchema to use 
        a dictionary as a datasource
    '''
        
    security = ClassSecurityInfo()                
    security.setDefaultAccess('allow')
    
    def __init__(self, dictionary):
        self.data = dictionary
        
    def __getitem__(self, key, default=_marker):
        v = self.data.get(key, default)
        if v is _marker:
            raise KeyError, key
        return v
    
    def __getattr__(self, key, default=_marker):
        try:
            return self.__getitem__(key, default)
        except KeyError:
            raise AttributeError, 'Cannot find ' + `key`
        
    
    
        
    


class PySchema(BaseSchema, Base, Implicit):
    """
        Python Based Schema
    """                
    security = ClassSecurityInfo()                
    security.setDefaultAccess('allow')
                 
    def __init__(self, fieldsets, i18n=None, context=None):
        '''
            fieldsets: a sequence of fieldsets in this schema
            i18n:      option callback function to translate text before returning
            context:   deprecated.
        '''
        self.context = context
        
        # Hash the fieldset Id's for easier retrieval later on, 
        # and keep a list of ids in the original order
        d_fieldsets = {}
        for fieldset in fieldsets:
            d_fieldsets[fieldset.Name()] = fieldset
            
        self.d_fieldsets = d_fieldsets            
        self.fieldsets   = [fieldset.Name() for fieldset in fieldsets]
        
        # Store the i18n callback
        self.i18n=i18n
    
    ###########################################################################        
            
    def i18n_translate(self, string, language=None):
        if callable( self.i18n ):
            return self.i18n( string, language )
        else:
            return string
      
    ###########################################################################              
    
    def bind(self, datasource, context=None):
        ''' 
            Binds this schema to a datasource (Any valid zope object)
            and returns a sequence of fieldsets
        '''
        
        # If we are passed a dictionary, bind to context
        if type(datasource) is dict:
            if not context:
                raise ValueError, \
                     'you must pass context to PySchema.bind() when binding to a dictionary'
                                 
            datasource = DictWrapper(datasource).__of__(context)
        
        #print "iter( %s ) is %s" % (`self`, `iter(self)`)
            
        #self.filter = filter        
        return self.__of__( datasource )
        
    
    def mapTo(self, *args, **kwargs):
        ''' 
            Deprecated; use bind instead
        '''
        return self.bind(*args,**kwargs) 
      

    def preprocess( self, form, datasource, context ): 
        
        for fieldset in self.bind( datasource, context=context ).getFieldsets():  
            for widget in fieldset.iterator():
                widget.preprocess( form )
    
    ###########################################################################
    
    def getFieldset(self, id):
        ''' Returns the fieldset with the specified id '''
        return self.d_fieldsets[id].__of__(self)
          
    def getFieldsets(self, filter=None):
        """ 
            Returns a sequence of fieldsets
            if filter is specified, the listed fieldsets will be returned IN THE SPECIFIED ORDER
            else, all fieldsets will be returned in the original order
         """
        if filter is None: filter = self.fieldsets
        
        return [self.getFieldset( fieldset ) for fieldset in filter]
        
    ###########################################################################        
    
    def allFields(self, fieldsets=None):
        for fieldset in self.getFieldsets( filter=fieldsets ):
            for widget in fieldset.getWidgets():
                yield widget.__of__(fieldset.__of__(self))
    
    def allFieldNames(self):
        """ Returns a list of all field names, in order """
        for widget in self.allFields():
            yield widget.Name() 
                
    def allFieldLabels(self):
        r = []
        for widget in self.allFields():
            r.append( (widget.Name(), widget.Label()) )
        return r 
        
    ###########################################################################        
    
    
    def asText( self, fieldsets=None ):
        
        out = StringIO()
        
        for field in self.allFields( fieldsets ):
            print >> out, field.asText()
            
        
        
        
        
        
    
from Globals import InitializeClass
InitializeClass(PySchema)
InitializeClass(DictWrapper)
