from AccessControl import ClassSecurityInfo
from Globals import InitializeClass
from Acquisition import Implicit
from Products.Hornet.CallbackIterator import CallbackIterator
from Products.Hornet.Schema.DisplayList import DisplayList

class BaseWidget(Implicit):
    security = ClassSecurityInfo()                
    security.setDefaultAccess('allow')
        
    # ZPT Macro to use (translates to here/widget/text/macros/[edit|view]
    widget = 'widget_text'
    
    def bind( self, datasource ):
        self.binding = datasource
        return self
    
    def getBinding( self ):
        return self.binding
    
    def macro(self, mode='edit'):
        return getattr(self, self.widget).macros[mode] 
    
    def __init__(self, key, **kwargs):
        self.key    = key
        self.kwargs = kwargs
        self.binding = None
        
    def Label(self):
        label = self.getArg('label', self.key )
        
        try:
            field_translation = self.i18n_translate( label )
        except KeyError:
            try:field_translation = self.i18n_translate( self.key )
            except KeyError:
                field_translation = label
        except AttributeError:
            return label
        return field_translation
            
    
    def Help(self):
        return self.getArg('help')
        
    def Description(self):
        desc = self.getArg('desc') or self.getArg('description') 
        try:return self.i18n_translate(desc)
        except KeyError:
            return desc
        
        
    def Name(self):
        return self.key
    
    def getCssClass(self):
        return self.getArg('css_class')
    
    def getArg(self, key, default=''):
        try:
            return self.kwargs[key]
        except KeyError:
            return default 
         
    def getWidgetValue(self, datasource=None):
        value = self.get( self.key, '' )
        if value is None: value = ''
        return value
        
    def get(self, key, default=None, datasource=None):
        if not datasource:
            datasource = self
            
        if hasattr( datasource, 'getValue'):    
            return datasource.getValue( key, default )
        else:
            return getattr( datasource, key, default )

    def Vocabulary(self, datasource=None, param='vocab'):
        
        #if hasattr(self, '_v_vocab_' + param):
        #    return getattr( self, '_v_vocab_' + param )
        
        vocab = self.getArg(param,{})
         
        if type( vocab ) is str:
                vocab = getattr( self, vocab )()
                #vocab = self.get(vocab, datasource=datasource)()

                
        #dl = DisplayList( vocab )
        
        #setattr( self, '_v_vocab_' + param, dl )
        
        #print "... " + self.Name() + " ... returning vocab " + `vocab`
        
        if not type( vocab ) is DisplayList:
            return DisplayList( vocab )
        else:
            return vocab 
                
        
                    
        
        
            
    def asText( self ):
        """
            Renders this widget as a plain text value (for emails, etc)
        """
        value = context.getWidgetValue()
        vocab = context.Vocabulary()
        
        v = vocab.get(value, None)
        if v:
            return v
        
        return value
            
        
        
        
            
            
               
        
            
        
        
        
            
    def preprocess( self, form ):
        """ 
            Perform any modifications to the form info.
            Modifies the dictionary passed to it. Does not return anything
        """              
        pass
    
            
InitializeClass(BaseWidget)