#####################################################################
#
#  Hornet SQL Bridge (C) 2005 Mooball.com pty ltd
#  A Zope product to mount an SQL Database into the ZODB
#
#  License: ZPL http://www.zope.org/Resources/ZPL
#  Contact: hornet@mooball.com
#
#  Please see http://plone.org/products/hornet
#
#  *WARNING: This product is ALPHA code. Do not expect it to work.*
#
#####################################################################

#####################################################################
#
#    WizardMixin
#    Author: James Davies (hornet@mooball.com)
#
#    Mixin class for Hornet Mount Point. Adds management functionality.
#
#####################################################################

from Products.PageTemplates.PageTemplateFile import PageTemplateFile 
from AccessControl import ClassSecurityInfo
from Globals import DTMLFile

class WizardMixin:
    """ 
        Mixin class; used to seperate management and wizard functionality 
        from core mountpoint functionality
    """
    
    manage_wizard    = PageTemplateFile('www/manage_wizard',globals())
    manage_mountForm = PageTemplateFile('www/manage_mountForm',globals())
    manage_template  = PageTemplateFile('www/manage_template',globals())
    
    tabs_js          = DTMLFile('skins/Hornet_widgets/tabs.js', globals())
    tabs_css         = DTMLFile('skins/Hornet_widgets/tabs.css', globals())
     
    security = ClassSecurityInfo()     
    
    def manage_mountTables(self, names, tables, primaryKeys, REQUEST=None):
        """ Mounts the specified tables """
        assert len(names) == len(tables) and len(tables) == len(primaryKeys), \
            "Form Error: Names, Tables, and primaryKeys are not the same length"
        
        addHornetFolder = self.manage_addProduct['Hornet'].manage_addHornetFolder
        
        i_tables = iter(tables)
        i_keys   = iter(primaryKeys)
        for name in names:
            table = i_tables.next()
            key   = i_keys.next()
            
            addHornetFolder(name, tableName=table, primaryKey=key)
            
        if REQUEST:
            return REQUEST.RESPONSE.redirect('manage_wizard')
        
    
    def guessTableDetails(self,table_name):
        """ Attempts to guess values for the folder wizard """
        details = self.sqlQuery('SHOW COLUMNS FROM %s' % table_name)
        results = {}
        
        # Iterate over the table schema and look for a primary key
        for row in details:
            if row['Key'] == 'PRI':
                results['primaryKey'] = row['Field']
                
        # Check for table name prefixes and drop if found (For the folder name)
        if table_name.count('_') is 1:
            prefix, name = table_name.split('_')
            if len(prefix) <= 4:
                results['name'] = name
                
        
        return results 
            
            
                
    security.declareProtected('Manage portal','manage_setConnectionId')
    def manage_setConnectionId(self, connectionId, REQUEST=None):
        """ Sets the default connection """
        self._connectionId = str(connectionId)
        if REQUEST:
            return REQUEST.RESPONSE.redirect('manage_wizard')
            
    security.declareProtected('Manage portal','getConnectionId')
    def getConnectionId(self):
        """ Returns the connection id """
        return self._connectionId        
        
    
    
    def getSQLFolders(self):
        """ Returns a list of available sql folders """
        for object in self.objectValues():
            if hasattr(object, 'isHornetFolder'):
                yield object
            
    def manage_getDBTableNames(self):
        rs = self.sqlQuery("SHOW TABLES")
            
        for row in rs:
            yield row.items()[0][1]
        
 
    
      