#####################################################################
#
#  Hornet SQL Bridge (C) 2005 Mooball.com pty ltd
#  A Zope product to mount an SQL Database into the ZODB
#
#  License: ZPL http://www.zope.org/Resources/ZPL
#  Contact: hornet@mooball.com
#
#  Please see http://plone.org/products/hornet
#
#  *WARNING: This product is ALPHA code. Do not expect it to work.*
#
#####################################################################

#####################################################################
#
#    ZopeSQLObject
#    Author: James Davies (hornet@mooball.com)
#
#    Zope specific methods of SQLObject
#
#####################################################################


##################
# Zope Imports   #
##################
from OFS.Traversable import Traversable
from OFS.SimpleItem import SimpleItem
from AccessControl import ClassSecurityInfo
from Acquisition import aq_base
from Products.CMFCore.utils import getToolByName

from Products.CMFCore.PortalContent import PortalContent
from Products.PageTemplates.PageTemplateFile import PageTemplateFile

##################
# Hornet Imports #
##################
from SQLTransactionAware import SQLTransactionAware
from SQLObject import SQLObject

from Products.Hornet.Schema.Sources.PySchema import PySchema
from Products.Hornet.Schema.Widgets import StringWidget, FieldSet

    
class ZopeSQLObject(Traversable, SQLObject, SimpleItem, SQLTransactionAware):
    """ 
        Adds Zope specific functionality to SQLObject
    """
    
    # Setup class security
    security = ClassSecurityInfo()
    security.setDefaultAccess('allow')
    security.declareObjectPublic()
    
    
    # Configure Tabs
    manage_options=(
        (
            {'label':'Edit', 'action':'manage_main',},
            {'label':'View', 'action':''}
        ))

    
    # Management Views
    manage_main = PageTemplateFile('www/manage_view', globals(), __name__='pt_baseEditForm')
    

    security.declareProtected('Modify portal content', 'manage_edit')
    def manage_edit(self, form=None, REQUEST=None, **kw):
        """ Merges the specified form information into this object """     
                
        
        if not form:
            if REQUEST:
                form = REQUEST.form
            else:
                form = kw
        
        
            
            
        
        # check if the object is new (i.e. not already in the database)
        #isNewObject = not ( self.data.get(self.getPrimaryKey(),False))
        isNewObject = not self.isExisting()
        
        schema = self.getSchema()
        if schema:
            form = dict( form )            
            schema.preprocess( form, datasource=self, context=self )
        
        for key, value in form.items():
            setattr(self, key, value)
            
        # If the call came directly from the browser...
        if REQUEST:
            
            # If the object is just being created, save it and grab the new id. 
            if isNewObject:
                insert_id = self.save()
                primaryKey = self.getPrimaryKey()
                self.set(self.getPrimaryKey(), insert_id)

            # Redirect to the objects url, (which should take into account the new id if relevant)
            return REQUEST.RESPONSE.redirect(REQUEST.get('came_from', self.absolute_url()))
                        
    def getSchema(self):
        """ Returns the schema for this object """        
        
        
        callback_name = getattr( self, 'cb_schema', None )
        if callback_name: 
            callback  = getattr( self, callback_name, None )
            if callback: 
                return callback()
        
        # No schema callback defined. Query the database.
                        
        table_name = self.getTableName()
        fields = [f['Field'] for f in self._v_mountpoint.describeTable( table_name )]
                
        widgets = []
        for field in fields:
            widgets.append( StringWidget( field ))
            
        fieldset = FieldSet( 'main', table_name, widgets )
        
        schema = PySchema( [fieldset] )
        
        return schema
            
            
            
            
    
    def getData(self):
        """ Return a sequence of brains for this object """
        for key, value in self._data.items():
            yield {'key': key, 'value': value}

    def getParentNode(self):
        """ Gets the parent node (aq_parent). Required by Plone Breadcrumbs """
        return self.aq_parent
    
    def getId(self):
        """Returns the value of the Primary Key """
        key = self._v_folder.getPrimaryKey()
        id = self.get( key )
        if not id: return "create"
        return str( id )
        

    
    def title_or_id(self):
        """ Returns the Title (if set), or the id """
        return self.Title() or self.getId()
    
    def title_and_id(self):
        return self.getId() + "( %s )" % self.Title()
        
    def Title(self):
        """ Returns the title """
        return ""
    
        
class CMFSQLObject(ZopeSQLObject, PortalContent):
    """ CMF Aware SQL Object """
    
    def listFolderContents(self, **kw):
        """ Don't reiterate over subobjects """
        return ()
    
    def updateDatabase(self):
        if not getattr(self, 'disable_catalog', None) == 'YES': 
            self.reindexObject() 
        ZopeSQLObject.updateDatabase(self) 
    
        
    def reindexObject(self, idxs=[]):
        if idxs == []:
        # Update the modification date.
            if hasattr(aq_base(self), 'notifyModified'):
                self.notifyModified()
        catalog = getToolByName(self._v_mountpoint, 'portal_catalog', None)
        if catalog is not None:
            catalog.reindexObject(self, idxs=idxs)

    
    objectValues = objectIds = objectItems = listFolderContents
        
    def getTypeInfo(self):
        """ Speed Patch
            Returns the correct type info object 
        """

        return self.portal_types.getTypeInfo(self.meta_type)

    
    def getIcon(self, relative_to_portal=0, cache={}):
        """ 
            Speed Patch 
            
            Plone calls this method umpteen million times for each page view, and 
            the default implementation is insanely slow. 
        """
                
        myicon = self.icon
        
        if relative_to_portal:
            if not 'portal_url' in cache: 
                portal_url = self.portal_url()
                cache['portal_url'] = portal_url
            else:
                portal_url = cache['portal_url']    
        
        
            return portal_url + '/' + myicon
            
        else:
            return myicon
        
        
        
        
        
